import { ActionReturnType, AsyncActionCallback, ParsedParameters } from './cli';
/**
 * A wrapper for the command action callback
 */
export declare class CLIAction {
    private readonly actionCallback;
    private _promiseInstance?;
    private _parsedParameters;
    private awaiterDone?;
    private static _runningAction?;
    /**
     * Create an instance of CLIAction
     * @param {AsyncActionCallback} actionCallback the handler method of the action
     */
    constructor(actionCallback: AsyncActionCallback);
    private get promiseInstance();
    private get parsedParameters();
    /**
     * A static member refering to the executed action
     * @name runningAction
     * @type {CLIAction}
     * @throws - When read before any action has run
     */
    static get runningAction(): CLIAction;
    /**
     * Triggers the callback of the action and returns its promise
     * @name trigger
     * @param {ParsedParameters} params - A key/value dict representing the parsed argv
     * @returns {Promise<ActionReturnType>} - The promise of the callback
     */
    trigger(params: ParsedParameters): Promise<ActionReturnType>;
    /**
     * A hook promise to the action callback
     * @returns {Promise<ParsedParameters} - Resolves into the parsed parameters; rejects if the callback throws, or if another action has run
     */
    actionAwaiter(): Promise<ParsedParameters>;
    private resolveAwaiter;
    private rejectAwaiter;
    /**
     * Rejects all the awaiters of this specific action with the given err
     * @param {Error} err - The error thrown while parsing the argv
     * @returns {void}
     */
    setParsingError(err: Error): void;
    /**
     * Rejects all the awaiters of this specific action because another action has run
     * @returns {void}
     */
    wasNotTriggered(): void;
}
