export declare type ParameterName = string;
export declare type ParameterType = 'single-value' | 'boolean' | 'array';
export declare type ParameterOverridenConfig = Partial<ParameterConfig> & Pick<ParameterConfig, 'name'>;
export interface ParameterConfig {
    name: ParameterName;
    aliases: ParameterName[];
    description: string;
    type?: ParameterType;
    default?: string;
    required?: boolean;
    forbiddenConjunctionParameters?: ParameterName[];
    requiredConjunctionParameters?: ParameterName[];
}
export declare class Parameter implements ParameterConfig {
    readonly name: string;
    private parameterData;
    private static parameters;
    constructor(arg: ParameterName | ParameterOverridenConfig);
    get aliases(): ParameterName[];
    get description(): string;
    get default(): string | undefined;
    get type(): ParameterType;
    get required(): boolean;
    get forbiddenParametersInConjunction(): ParameterName[];
    get requiredParametersInConjunction(): ParameterName[];
    static declare(parameter: ParameterConfig): void;
    /**
     * @name reset
     * For testing purposes only. It will just remove all parameters declaration
     * @returns {ParameterConfig[]} the removed parameters
     */
    static reset(): ParameterConfig[];
    static get(parameterName: ParameterName): ParameterConfig;
}
