"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Parameter = void 0;
class Parameter {
    constructor(arg) {
        const argAsParameterName = arg;
        const argAsOverridenConfig = arg;
        const overridenConfig = (function () {
            if (typeof arg === 'string') {
                return undefined;
            }
            else {
                return argAsOverridenConfig;
            }
        })();
        this.name = (overridenConfig === null || overridenConfig === void 0 ? void 0 : overridenConfig.name) || argAsParameterName;
        this.parameterData = Object.assign(Parameter.get(this.name), overridenConfig);
    }
    get aliases() {
        return this.parameterData.aliases;
    }
    get description() {
        return this.parameterData.description;
    }
    get default() {
        return this.parameterData.default;
    }
    get type() {
        return this.parameterData.type || 'single-value';
    }
    get required() {
        return !!this.parameterData.required;
    }
    get forbiddenParametersInConjunction() {
        return Array.isArray(this.parameterData.forbiddenConjunctionParameters)
            ? this.parameterData.forbiddenConjunctionParameters.slice()
            : [];
    }
    get requiredParametersInConjunction() {
        return Array.isArray(this.parameterData.requiredConjunctionParameters)
            ? this.parameterData.requiredConjunctionParameters.slice()
            : [];
    }
    static declare(parameter) {
        Parameter.parameters.push(parameter);
    }
    /**
     * @name reset
     * For testing purposes only. It will just remove all parameters declaration
     * @returns {ParameterConfig[]} the removed parameters
     */
    static reset() {
        return this.parameters.splice(0, this.parameters.length);
    }
    static get(parameterName) {
        const param = Parameter.parameters.find((p) => p.name === parameterName);
        if (!param) {
            throw new Error(`No such parameter ${parameterName}`);
        }
        // It uses Object.assign in order to return a copy of the instance instead of the original one, so modifying it won't affect the original
        return Object.assign({}, param);
    }
}
exports.Parameter = Parameter;
Parameter.parameters = [];
