"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ParametersHelper = void 0;
const fs = __importStar(require("fs"));
const parameter_declarations_1 = require("../parameter_declarations");
const __1 = require("..");
const prompts_1 = __importDefault(require("prompts"));
const ardrive_core_js_1 = require("ardrive-core-js");
const ipfs_utils_1 = require("../utils/ipfs_utils");
const DEFAULT_GATEWAY = 'https://arweave.net:443';
const ARWEAVE_GATEWAY_ENV_VAR = 'ARWEAVE_GATEWAY';
/**
 * @type {ParametersHelper}
 * A class that assists with handling Commander options during common ArDrive CLI workflows
 */
class ParametersHelper {
    /**
     * @returns {ParametersHelper}
     * @param {any} options The object containing the parameterName: value mapping
     * An immutable instance of ParametersHelper holding the parsed values of the parameters
     */
    constructor(options, walletDao = __1.cliWalletDao) {
        this.options = options;
        this.walletDao = walletDao;
    }
    /**
     * @returns {Promise<boolean>}
     * Returns true when a drive password or drive key is provided
     */
    getIsPrivate() {
        return __awaiter(this, void 0, void 0, function* () {
            return (this.getParameterValue(parameter_declarations_1.PrivateParameter) !== undefined ||
                this.getParameterValue(parameter_declarations_1.UnsafeDrivePasswordParameter) !== undefined ||
                this.getParameterValue(parameter_declarations_1.DriveKeyParameter) !== undefined);
        });
    }
    /**
     * @returns {Promise<Wallet>}
     * Will return a wallet instance created from the seed phrase or the walletFile.
     * Throws an error if a wallet can't be created.
     */
    getRequiredWallet() {
        return __awaiter(this, void 0, void 0, function* () {
            const walletFile = this.getParameterValue(parameter_declarations_1.WalletFileParameter);
            const seedPhrase = this.getParameterValue(parameter_declarations_1.SeedPhraseParameter);
            if (walletFile) {
                const walletFileData = fs.readFileSync(walletFile, { encoding: 'utf8', flag: 'r' });
                const walletJSON = JSON.parse(walletFileData);
                const walletJWK = walletJSON;
                return new ardrive_core_js_1.JWKWallet(walletJWK);
            }
            else if (seedPhrase) {
                return yield this.walletDao.generateJWKWallet(new ardrive_core_js_1.SeedPhrase(seedPhrase));
            }
            throw new Error('Neither a wallet file nor seed phrase was provided!');
        });
    }
    getOptionalWallet() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getRequiredWallet().catch(() => null);
        });
    }
    getWalletAddress() {
        return __awaiter(this, void 0, void 0, function* () {
            const address = this.getParameterValue(parameter_declarations_1.AddressParameter);
            if (address) {
                return ardrive_core_js_1.ADDR(address);
            }
            return this.getRequiredWallet().then((wallet) => wallet.getAddress());
        });
    }
    getOptionalBoostSetting() {
        const boost = this.getParameterValue(parameter_declarations_1.BoostParameter);
        return boost ? new ardrive_core_js_1.FeeMultiple(+boost) : undefined;
    }
    getPrivateKeyData() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            // Gather optional private parameters
            const driveKey = this.getParameterValue(parameter_declarations_1.DriveKeyParameter);
            const wallet = yield this.getOptionalWallet();
            const password = yield (() => {
                if (
                // If private param specified or an unsafe password param is provided
                this.getParameterValue(parameter_declarations_1.PrivateParameter) !== undefined ||
                    this.getParameterValue(parameter_declarations_1.UnsafeDrivePasswordParameter) !== undefined) {
                    return this.getDrivePassword();
                }
                return Promise.resolve(undefined);
            })();
            return new ardrive_core_js_1.PrivateKeyData({
                password,
                driveKeys: driveKey ? [new ardrive_core_js_1.EntityKey(Buffer.from(driveKey, 'base64'))] : undefined,
                wallet: (_a = wallet) !== null && _a !== void 0 ? _a : undefined
            });
        });
    }
    getDriveKey({ driveId, drivePassword, useCache = false }) {
        return __awaiter(this, void 0, void 0, function* () {
            // Obtain drive key from one of:
            // • --drive-key param
            // • (--wallet-file or --seed-phrase) + (--unsafe-drive-password or --private password)
            if (useCache) {
                const cachedDriveKey = ParametersHelper.driveKeyCache[`${driveId}`];
                if (cachedDriveKey) {
                    return cachedDriveKey;
                }
            }
            const driveKey = this.getParameterValue(parameter_declarations_1.DriveKeyParameter);
            if (driveKey) {
                const paramDriveKey = new ardrive_core_js_1.EntityKey(Buffer.from(driveKey, 'base64'));
                ParametersHelper.driveKeyCache[`${driveId}`] = paramDriveKey;
                return paramDriveKey;
            }
            drivePassword = drivePassword !== null && drivePassword !== void 0 ? drivePassword : (yield this.getDrivePassword());
            if (drivePassword) {
                const wallet = (yield this.getRequiredWallet());
                const derivedDriveKey = yield ardrive_core_js_1.deriveDriveKey(drivePassword, `${driveId}`, JSON.stringify(wallet.getPrivateKey()));
                ParametersHelper.driveKeyCache[`${driveId}`] = derivedDriveKey;
                return derivedDriveKey;
            }
            throw new Error(`No drive key or password provided for drive ID ${driveId}!`);
        });
    }
    getDrivePassword(isForNewDrive = false) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.getParameterValue(parameter_declarations_1.PrivateParameter)) {
                // Try to get password from STDIN, then ENV.ARDRIVE_DRIVE_PW, then interactive secure prompt
                try {
                    const stdInPassword = fs.readFileSync(process.stdin.fd).toString().replace(/\n*$/, '');
                    if (stdInPassword) {
                        return stdInPassword;
                    }
                }
                catch (_err) {
                    // Do nothing
                }
                const envPassword = process.env['ARDRIVE_DRIVE_PW'];
                if (envPassword) {
                    return envPassword;
                }
                const promptedPassword = yield prompts_1.default({
                    type: 'text',
                    name: 'password',
                    style: 'password',
                    message: isForNewDrive ? 'Enter new drive password:' : 'Enter drive password:'
                });
                if (isForNewDrive) {
                    const confirmedPassword = yield prompts_1.default({
                        type: 'text',
                        name: 'password',
                        style: 'password',
                        message: 'Re-enter new drive password: '
                    });
                    if (confirmedPassword.password !== promptedPassword.password) {
                        throw new Error('Drive passwords do not match!');
                    }
                }
                if (!promptedPassword.password.length) {
                    throw new Error('New drive password must not be empty when --private is specified!');
                }
                return promptedPassword.password;
            }
            const unsafePassword = this.getParameterValue(parameter_declarations_1.UnsafeDrivePasswordParameter);
            if (!unsafePassword) {
                throw new Error('Password not detected for private drive operation! Please provide a password via the --private option (recommended) or the --unsafe-drive-password option (not recommended).');
            }
            return unsafePassword;
        });
    }
    getMaxDepth(defaultDepth = 0) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (this.getParameterValue(parameter_declarations_1.AllParameter)) {
                return Number.MAX_SAFE_INTEGER;
            }
            const maxDepthValue = Number((_a = this.getParameterValue(parameter_declarations_1.MaxDepthParameter)) !== null && _a !== void 0 ? _a : defaultDepth);
            if (!Number.isInteger(maxDepthValue) || maxDepthValue < 0) {
                throw new Error('maxDepth should be a non-negative integer!');
            }
            return maxDepthValue;
        });
    }
    getFileNameConflictResolution() {
        if (this.getParameterValue(parameter_declarations_1.ReplaceParameter)) {
            return ardrive_core_js_1.replaceOnConflicts;
        }
        if (this.getParameterValue(parameter_declarations_1.SkipParameter)) {
            return ardrive_core_js_1.skipOnConflicts;
        }
        if (this.getParameterValue(parameter_declarations_1.AskParameter)) {
            return ardrive_core_js_1.askOnConflicts;
        }
        return ardrive_core_js_1.upsertOnConflicts;
    }
    /**
     * @param {ParameterName} parameterName
     * @returns {string | undefined}
     * Returns the string value for the specific parameter; returns undefined if not set
     */
    getParameterValue(parameterName, 
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    mapFunc = (input) => input) {
        const value = this.options[parameterName];
        return value ? mapFunc(value) : value;
    }
    /**
     * @param {ParameterName} parameterName
     * @param {(input: any) => T} mapFunc A function that maps the parameter value into a T instance
     * @returns {string | undefined}
     * @throws - When the required parameter value has a falsy value
     * Returns the string value for the specific parameter
     */
    getRequiredParameterValue(parameterName, 
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    mapFunc = (input) => input) {
        const value = this.options[parameterName];
        if (!value) {
            throw new Error(`Required parameter ${parameterName} wasn't provided!`);
        }
        return mapFunc(value);
    }
    getCustomMetaDataWithIpfsCid({ localFilePath }) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const customMetaDataClone = Object.assign({}, this.getCustomMetaData());
            const customIpfsTag = (_a = customMetaDataClone.dataGqlTags) === null || _a === void 0 ? void 0 : _a['IPFS-Add'];
            if (customIpfsTag) {
                throw new Error(`You cannot pass the --add-ipfs-tag flag and set the custom IPFS-Add metadata item. Found: { 'IPFS-Add': ${JSON.stringify(customIpfsTag)}}`);
            }
            else {
                const fileContent = fs.readFileSync(localFilePath);
                const cidHash = yield ipfs_utils_1.deriveIpfsCid(fileContent);
                if (!customMetaDataClone.dataGqlTags) {
                    customMetaDataClone.dataGqlTags = {};
                }
                customMetaDataClone.dataGqlTags['IPFS-Add'] = cidHash;
            }
            return customMetaDataClone;
        });
    }
    getCustomMetaData() {
        const metaDataPath = this.getParameterValue(parameter_declarations_1.MetaDataFileParameter);
        const customMetaData = (() => {
            if (metaDataPath) {
                return this.readMetaDataFromPath(metaDataPath);
            }
            const customMetaData = {};
            const metaDataGqlTags = this.mapMetaDataArrayToCustomMetaDataShape(this.getParameterValue(parameter_declarations_1.MetaDataGqlTagsParameter));
            if (metaDataGqlTags) {
                Object.assign(customMetaData, { metaDataGqlTags });
            }
            const dataGqlTags = this.mapMetaDataArrayToCustomMetaDataShape(this.getParameterValue(parameter_declarations_1.DataGqlTagsParameter));
            if (dataGqlTags) {
                Object.assign(customMetaData, { dataGqlTags });
            }
            const metaDataJson = this.getParameterValue(parameter_declarations_1.MetadataJsonParameter);
            if (metaDataJson) {
                Object.assign(customMetaData, { metaDataJson: this.parseMetaDataJson(metaDataJson) });
            }
            return customMetaData;
        })();
        if (Object.keys(customMetaData).length > 0) {
            ardrive_core_js_1.assertCustomMetaData(customMetaData);
            return customMetaData;
        }
        return undefined;
    }
    readMetaDataFromPath(path) {
        // Read tag file or throw fs path error or throw JSON parse error
        const tagFile = fs.readFileSync(path, { encoding: 'utf8' });
        return JSON.parse(tagFile);
    }
    mapMetaDataArrayToCustomMetaDataShape(metadata) {
        if (metadata === undefined || metadata.length === 0) {
            return undefined;
        }
        this.assertEvenTags(metadata);
        const metaData = {};
        let temp = null;
        for (const val of metadata) {
            if (temp === null) {
                // val is tag Name
                temp = val;
            }
            else {
                // val is tag Value
                metaData[temp] = val;
                temp = null;
            }
        }
        return metaData;
    }
    assertEvenTags(tags) {
        if (tags && tags.length % 2 !== 0) {
            throw Error('User must provide an even number custom metadata inputs! e.g: --metadata-json "NAME" "VALUE"');
        }
    }
    parseMetaDataJson(rawMetaDataJsonString) {
        return JSON.parse(rawMetaDataJsonString);
    }
    /**
     * Gathers a valid gateway URL from user provided gateway parameter,
     * an environment variable, or returns the default arweave gateway
     *
     * @throws on user provided gateways that are incompatible with URL class constructor
     * @throws when hostName cannot be derived from a user provided gateway
     */
    getGateway() {
        const userProvidedURL = (() => {
            // Use optional --gateway supplied parameter as first choice
            const gatewayFromParam = this.getParameterValue(parameter_declarations_1.GatewayParameter);
            if (gatewayFromParam) {
                return new URL(gatewayFromParam);
            }
            // Then check for an ENV provided gateway
            const envGateway = process.env[ARWEAVE_GATEWAY_ENV_VAR];
            if (envGateway) {
                return new URL(envGateway);
            }
            return undefined;
        })();
        if (!userProvidedURL) {
            // Return default CLI Arweave if no gateway can be derived from the user
            return new URL(DEFAULT_GATEWAY);
        }
        if (userProvidedURL.hostname === '') {
            // Ensure a valid host name was provided to be used in Arweave.init()
            throw new TypeError(`Host name could not be determined from provided URL: ${userProvidedURL.href}`);
        }
        return userProvidedURL;
    }
    isDryRun() {
        const dryRun = this.getParameterValue(parameter_declarations_1.DryRunParameter);
        return !!dryRun;
    }
}
exports.ParametersHelper = ParametersHelper;
ParametersHelper.driveKeyCache = {};
