"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ardrive_core_js_1 = require("ardrive-core-js");
const __1 = require("..");
const CLICommand_1 = require("../CLICommand");
const action_1 = require("../CLICommand/action");
const error_codes_1 = require("../CLICommand/error_codes");
const parameter_declarations_1 = require("../parameter_declarations");
const prompts_1 = require("../prompts");
const get_arweave_for_url_1 = require("../utils/get_arweave_for_url");
new CLICommand_1.CLICommand({
    name: 'create-manifest',
    parameters: [
        parameter_declarations_1.FolderIdParameter,
        parameter_declarations_1.DestinationManifestNameParameter,
        parameter_declarations_1.BoostParameter,
        parameter_declarations_1.DryRunParameter,
        parameter_declarations_1.WalletFileParameter,
        parameter_declarations_1.SeedPhraseParameter,
        ...parameter_declarations_1.ConflictResolutionParams,
        ...parameter_declarations_1.TreeDepthParams,
        parameter_declarations_1.GatewayParameter
    ],
    action: new action_1.CLIAction(function action(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const parameters = new CLICommand_1.ParametersHelper(options, __1.cliWalletDao);
            const wallet = yield parameters.getRequiredWallet();
            const arweave = get_arweave_for_url_1.getArweaveFromURL(parameters.getGateway());
            const arDrive = __1.cliArDriveFactory({
                wallet: wallet,
                feeMultiple: parameters.getOptionalBoostSetting(),
                dryRun: parameters.isDryRun(),
                arweave
            });
            const folderId = parameters.getRequiredParameterValue(parameter_declarations_1.FolderIdParameter, ardrive_core_js_1.EID);
            const maxDepth = yield parameters.getMaxDepth(Number.MAX_SAFE_INTEGER);
            const destManifestName = parameters.getParameterValue(parameter_declarations_1.DestinationManifestNameParameter);
            const conflictResolution = parameters.getFileNameConflictResolution();
            const result = yield arDrive.uploadPublicManifest({
                folderId: folderId,
                maxDepth,
                destManifestName,
                conflictResolution,
                prompts: prompts_1.fileUploadConflictPrompts
            });
            console.log(JSON.stringify(result, null, 4));
            return error_codes_1.SUCCESS_EXIT_CODE;
        });
    })
});
