"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ardrive_core_js_1 = require("ardrive-core-js");
const __1 = require("..");
const CLICommand_1 = require("../CLICommand");
const CLICommand_2 = require("../CLICommand");
const action_1 = require("../CLICommand/action");
const error_codes_1 = require("../CLICommand/error_codes");
const parameter_declarations_1 = require("../parameter_declarations");
new CLICommand_1.CLICommand({
    name: 'create-tx',
    parameters: [
        parameter_declarations_1.ArAmountParameter,
        parameter_declarations_1.DestinationAddressParameter,
        parameter_declarations_1.RewardParameter,
        parameter_declarations_1.LastTxParameter,
        ...parameter_declarations_1.WalletTypeParameters
    ],
    action: new action_1.CLIAction(function action(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const parameters = new CLICommand_2.ParametersHelper(options);
            const arAmount = parameters.getRequiredParameterValue(parameter_declarations_1.ArAmountParameter, ardrive_core_js_1.AR.from);
            const winston = arAmount.toWinston();
            const destAddress = parameters.getRequiredParameterValue(parameter_declarations_1.DestinationAddressParameter, ardrive_core_js_1.ADDR);
            const jwkWallet = (yield parameters.getRequiredWallet());
            const lastTxParam = parameters.getParameterValue(parameter_declarations_1.LastTxParameter); // Can be provided as a txID or empty string
            const last_tx = lastTxParam && lastTxParam.length ? `${ardrive_core_js_1.TxID(lastTxParam)}` : undefined;
            // Create and sign transaction
            const trxAttributes = {
                target: destAddress.toString(),
                quantity: winston.toString(),
                reward: `${parameters.getRequiredParameterValue(parameter_declarations_1.RewardParameter, ardrive_core_js_1.W)}`,
                last_tx
            };
            const transaction = yield __1.cliArweave.createTransaction(trxAttributes, jwkWallet.getPrivateKey());
            transaction.addTag('App-Name', __1.CLI_APP_NAME);
            transaction.addTag('App-Version', __1.CLI_APP_VERSION);
            transaction.addTag('Type', 'transfer');
            yield __1.cliArweave.transactions.sign(transaction, jwkWallet.getPrivateKey());
            console.log(JSON.stringify(transaction));
            return error_codes_1.SUCCESS_EXIT_CODE;
        });
    })
});
