"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ardrive_core_js_1 = require("ardrive-core-js");
const index_1 = require("../index");
const CLICommand_1 = require("../CLICommand");
const action_1 = require("../CLICommand/action");
const parameter_declarations_1 = require("../parameter_declarations");
const utils_1 = require("../utils");
const path_1 = require("path");
const get_arweave_for_url_1 = require("../utils/get_arweave_for_url");
new CLICommand_1.CLICommand({
    name: 'download-drive',
    parameters: [
        parameter_declarations_1.DriveIdParameter,
        {
            name: parameter_declarations_1.LocalPathParameter,
            description: '(OPTIONAL) the path on the local filesystem where the folder should be created and into which the contents of the drive are then downloaded. By default, the folder is created in the current working directory.'
        },
        parameter_declarations_1.MaxDepthParameter,
        ...parameter_declarations_1.DrivePrivacyParameters,
        parameter_declarations_1.GatewayParameter
    ],
    action: new action_1.CLIAction((options) => __awaiter(void 0, void 0, void 0, function* () {
        const parameters = new CLICommand_1.ParametersHelper(options);
        const driveId = parameters.getRequiredParameterValue(parameter_declarations_1.DriveIdParameter, ardrive_core_js_1.EID);
        const maxDepth = yield parameters.getMaxDepth(Number.MAX_SAFE_INTEGER);
        const destOutputPath = parameters.getParameterValue(parameter_declarations_1.LocalPathParameter) || '.';
        const [destFolderPath, customFolderName] = utils_1.getOutputFolderPathAndName(destOutputPath);
        let outputPath;
        const gateway = parameters.getGateway();
        const arweave = get_arweave_for_url_1.getArweaveFromURL(gateway);
        if (yield parameters.getIsPrivate()) {
            const wallet = yield parameters.getRequiredWallet();
            const ardrive = index_1.cliArDriveFactory({
                wallet,
                arweave
            });
            const driveKey = yield parameters.getDriveKey({ driveId });
            yield ardrive.downloadPrivateDrive({
                driveId,
                driveKey,
                destFolderPath,
                customFolderName,
                maxDepth
            });
            outputPath = path_1.join(destFolderPath, customFolderName ? customFolderName : (yield ardrive.getPrivateDrive({ driveId, driveKey })).name);
        }
        else {
            const ardrive = index_1.cliArDriveAnonymousFactory({ arweave });
            yield ardrive.downloadPublicDrive({ driveId, destFolderPath, customFolderName, maxDepth });
            outputPath = path_1.join(destFolderPath, customFolderName ? customFolderName : (yield ardrive.getPublicDrive({ driveId })).name);
        }
        console.log(`Drive with ID "${driveId}" was successfully downloaded to "${outputPath}"`);
    }))
});
