"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ardrive_core_js_1 = require("ardrive-core-js");
const index_1 = require("../index");
const CLICommand_1 = require("../CLICommand");
const action_1 = require("../CLICommand/action");
const parameter_declarations_1 = require("../parameter_declarations");
const utils_1 = require("../utils");
const path_1 = require("path");
const get_arweave_for_url_1 = require("../utils/get_arweave_for_url");
new CLICommand_1.CLICommand({
    name: 'download-file',
    parameters: [
        parameter_declarations_1.FileIdParameter,
        {
            name: parameter_declarations_1.LocalPathParameter,
            description: '(OPTIONAL) the path on the local filesystem where the file should be downloaded. Defaults to current working directory.'
        },
        ...parameter_declarations_1.DrivePrivacyParameters,
        parameter_declarations_1.GatewayParameter
    ],
    action: new action_1.CLIAction((options) => __awaiter(void 0, void 0, void 0, function* () {
        const parameters = new CLICommand_1.ParametersHelper(options);
        const fileId = parameters.getRequiredParameterValue(parameter_declarations_1.FileIdParameter, ardrive_core_js_1.EID);
        const destOutputPath = parameters.getParameterValue(parameter_declarations_1.LocalPathParameter) || '.';
        const arweave = get_arweave_for_url_1.getArweaveFromURL(parameters.getGateway());
        const [destFolderPath, defaultFileName] = utils_1.getOutputFilePathAndName(destOutputPath);
        let outputPath;
        if (yield parameters.getIsPrivate()) {
            const wallet = yield parameters.getRequiredWallet();
            const ardrive = index_1.cliArDriveFactory({
                wallet,
                feeMultiple: parameters.getOptionalBoostSetting(),
                arweave
            });
            const driveId = yield ardrive.getDriveIdForFileId(fileId);
            const driveKey = yield parameters.getDriveKey({ driveId: driveId });
            yield ardrive.downloadPrivateFile({ fileId, driveKey, destFolderPath, defaultFileName });
            outputPath = path_1.join(destFolderPath, defaultFileName ? defaultFileName : (yield ardrive.getPrivateFile({ fileId, driveKey })).name);
        }
        else {
            const ardrive = index_1.cliArDriveAnonymousFactory({ arweave });
            yield ardrive.downloadPublicFile({ fileId, destFolderPath, defaultFileName });
            outputPath = path_1.join(destFolderPath, defaultFileName ? defaultFileName : (yield ardrive.getPublicFile({ fileId })).name);
        }
        console.log(`File with ID "${fileId}" was successfully downloaded to "${outputPath}"`);
    }))
});
