"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const CLICommand_1 = require("../CLICommand");
const parameter_declarations_1 = require("../parameter_declarations");
const __1 = require("..");
const error_codes_1 = require("../CLICommand/error_codes");
const action_1 = require("../CLICommand/action");
const ardrive_core_js_1 = require("ardrive-core-js");
const get_arweave_for_url_1 = require("../utils/get_arweave_for_url");
new CLICommand_1.CLICommand({
    name: 'folder-info',
    parameters: [parameter_declarations_1.FolderIdParameter, parameter_declarations_1.GetAllRevisionsParameter, ...parameter_declarations_1.DrivePrivacyParameters, parameter_declarations_1.GatewayParameter],
    action: new action_1.CLIAction(function action(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const parameters = new CLICommand_1.ParametersHelper(options);
            const arweave = get_arweave_for_url_1.getArweaveFromURL(parameters.getGateway());
            const result = yield (function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const folderId = ardrive_core_js_1.EID(parameters.getRequiredParameterValue(parameter_declarations_1.FolderIdParameter));
                    if (yield parameters.getIsPrivate()) {
                        const wallet = yield parameters.getRequiredWallet();
                        const arDrive = __1.cliArDriveFactory({ wallet, arweave });
                        const driveId = yield arDrive.getDriveIdForFolderId(folderId);
                        const driveKey = yield parameters.getDriveKey({ driveId });
                        // We have the drive id from deriving a key, we can derive the owner
                        const driveOwner = yield arDrive.getOwnerForDriveId(driveId);
                        return arDrive.getPrivateFolder({ folderId, driveKey, owner: driveOwner });
                    }
                    else {
                        const arDrive = __1.cliArDriveAnonymousFactory({ arweave });
                        return arDrive.getPublicFolder({ folderId });
                    }
                });
            })();
            // TODO: Fix base types so deleting un-used values is not necessary; Tickets: PE-525 + PE-556
            delete result.size;
            delete result.dataTxId;
            delete result.dataContentType;
            delete result.lastModifiedDate;
            console.log(JSON.stringify(result, null, 4));
            return error_codes_1.SUCCESS_EXIT_CODE;
        });
    })
});
