"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ardrive_core_js_1 = require("ardrive-core-js");
const CLICommand_1 = require("../CLICommand");
const action_1 = require("../CLICommand/action");
const parameter_declarations_1 = require("../parameter_declarations");
const get_arweave_for_url_1 = require("../utils/get_arweave_for_url");
new CLICommand_1.CLICommand({
    name: 'get-file-key',
    parameters: [
        ...parameter_declarations_1.DriveCreationPrivacyParameters,
        parameter_declarations_1.DriveIdParameter,
        parameter_declarations_1.DriveKeyParameter,
        parameter_declarations_1.FileIdParameter,
        parameter_declarations_1.NoVerifyParameter,
        parameter_declarations_1.GatewayParameter
    ],
    action: new action_1.CLIAction(function action(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const parameters = new CLICommand_1.ParametersHelper(options);
            const fileId = ardrive_core_js_1.EID(parameters.getRequiredParameterValue(parameter_declarations_1.FileIdParameter));
            // Obviate the need for a drive ID when a drive key is specified
            const driveKey = yield (() => __awaiter(this, void 0, void 0, function* () {
                const driveKeyParam = parameters.getParameterValue(parameter_declarations_1.DriveKeyParameter);
                if (driveKeyParam) {
                    return new ardrive_core_js_1.EntityKey(Buffer.from(driveKeyParam, 'base64'));
                }
                // Lean on getDriveKey with a specified driveID
                // TODO: In the future, loosen driveID requirement and fetch from fileID
                const driveId = ardrive_core_js_1.EID(parameters.getRequiredParameterValue(parameter_declarations_1.DriveIdParameter));
                return yield parameters.getDriveKey({ driveId });
            }))();
            const fileKey = yield ardrive_core_js_1.deriveFileKey(`${fileId}`, driveKey);
            if (options.verify) {
                const arweave = get_arweave_for_url_1.getArweaveFromURL(parameters.getGateway());
                const file = yield new ardrive_core_js_1.ArFSPrivateFileBuilder(fileId, new ardrive_core_js_1.GatewayAPI({ gatewayUrl: ardrive_core_js_1.gatewayUrlForArweave(arweave) }), driveKey, undefined, fileKey).build();
                console.log(file.fileKey.toJSON());
            }
            else {
                console.log(fileKey.toJSON());
            }
        });
    })
});
