"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ardrive_core_js_1 = require("ardrive-core-js");
const __1 = require("..");
const CLICommand_1 = require("../CLICommand");
const action_1 = require("../CLICommand/action");
const error_codes_1 = require("../CLICommand/error_codes");
const parameter_declarations_1 = require("../parameter_declarations");
const get_arweave_for_url_1 = require("../utils/get_arweave_for_url");
new CLICommand_1.CLICommand({
    name: 'list-drive',
    parameters: [parameter_declarations_1.DriveIdParameter, parameter_declarations_1.WithKeysParameter, ...parameter_declarations_1.TreeDepthParams, ...parameter_declarations_1.DrivePrivacyParameters, parameter_declarations_1.GatewayParameter],
    action: new action_1.CLIAction(function action(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const parameters = new CLICommand_1.ParametersHelper(options, __1.cliWalletDao);
            const driveId = parameters.getRequiredParameterValue(parameter_declarations_1.DriveIdParameter, ardrive_core_js_1.EID);
            let children;
            const maxDepth = yield parameters.getMaxDepth(Number.MAX_SAFE_INTEGER);
            const arweave = get_arweave_for_url_1.getArweaveFromURL(parameters.getGateway());
            if (yield parameters.getIsPrivate()) {
                const wallet = yield parameters.getRequiredWallet();
                const arDrive = __1.cliArDriveFactory({ wallet, arweave });
                const driveKey = yield parameters.getDriveKey({ driveId });
                const drive = yield arDrive.getPrivateDrive({ driveId, driveKey });
                const rootFolderId = drive.rootFolderId;
                const withKeys = yield parameters.getParameterValue(parameter_declarations_1.WithKeysParameter, (value) => !!value);
                // We have the drive id from deriving a key, we can derive the owner
                const driveOwner = yield arDrive.getOwnerForDriveId(driveId);
                children = yield arDrive.listPrivateFolder({
                    folderId: rootFolderId,
                    driveKey,
                    maxDepth,
                    includeRoot: true,
                    owner: driveOwner,
                    withKeys
                });
            }
            else {
                const arDrive = __1.cliArDriveAnonymousFactory({ arweave });
                const drive = yield arDrive.getPublicDrive({ driveId });
                const rootFolderId = drive.rootFolderId;
                children = yield arDrive.listPublicFolder({ folderId: rootFolderId, maxDepth, includeRoot: true });
            }
            const sortedChildren = children.sort((a, b) => ardrive_core_js_1.alphabeticalOrder(a.path, b.path));
            // TODO: Fix base types so deleting un-used values is not necessary; Tickets: PE-525 + PE-556
            sortedChildren.map((fileOrFolderMetaData) => {
                if (fileOrFolderMetaData.entityType === 'folder') {
                    delete fileOrFolderMetaData.lastModifiedDate;
                    delete fileOrFolderMetaData.size;
                    delete fileOrFolderMetaData.dataTxId;
                    delete fileOrFolderMetaData.dataContentType;
                }
            });
            // Display data
            console.log(JSON.stringify(sortedChildren, null, 4));
            return error_codes_1.SUCCESS_EXIT_CODE;
        });
    })
});
