"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const CLICommand_1 = require("../CLICommand");
const parameter_declarations_1 = require("../parameter_declarations");
const error_codes_1 = require("../CLICommand/error_codes");
const action_1 = require("../CLICommand/action");
const ardrive_core_js_1 = require("ardrive-core-js");
const __1 = require("..");
const get_arweave_for_url_1 = require("../utils/get_arweave_for_url");
new CLICommand_1.CLICommand({
    name: 'retry-tx',
    parameters: [
        parameter_declarations_1.LocalPathParameter,
        { name: parameter_declarations_1.ParentFolderIdParameter, required: false },
        { name: parameter_declarations_1.FileIdParameter, required: false },
        parameter_declarations_1.DestinationFileNameParameter,
        parameter_declarations_1.BoostParameter,
        parameter_declarations_1.DryRunParameter,
        ...parameter_declarations_1.ConflictResolutionParams,
        parameter_declarations_1.CustomContentTypeParameter,
        parameter_declarations_1.GatewayParameter,
        {
            name: parameter_declarations_1.TransactionIdParameter,
            description: '(PUBLIC UNBUNDLED FILES ONLY) The transaction ID of the data transaction to retry'
        },
        ...parameter_declarations_1.WalletTypeParameters
    ],
    action: new action_1.CLIAction(function action(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const parameters = new CLICommand_1.ParametersHelper(options);
            const conflictResolution = parameters.getFileNameConflictResolution();
            const customContentType = parameters.getParameterValue(parameter_declarations_1.CustomContentTypeParameter);
            const localFilePath = parameters.getRequiredParameterValue(parameter_declarations_1.LocalPathParameter);
            const dataTxId = parameters.getRequiredParameterValue(parameter_declarations_1.TransactionIdParameter, ardrive_core_js_1.TxID);
            const wrappedFile = ardrive_core_js_1.wrapFileOrFolder(localFilePath, customContentType);
            if (wrappedFile.entityType !== 'file') {
                throw Error('Retrying folder uploads is not yet supported!');
            }
            const wallet = yield parameters.getRequiredWallet();
            const arweave = get_arweave_for_url_1.getArweaveFromURL(parameters.getGateway());
            const arDrive = __1.cliArDriveFactory({
                wallet,
                feeMultiple: parameters.getOptionalBoostSetting(),
                dryRun: parameters.isDryRun(),
                arweave
            });
            const destinationFolderId = parameters.getParameterValue(parameter_declarations_1.ParentFolderIdParameter, ardrive_core_js_1.EID);
            const fileId = parameters.getParameterValue(parameter_declarations_1.FileIdParameter, ardrive_core_js_1.EID);
            const results = yield (() => __awaiter(this, void 0, void 0, function* () {
                if (fileId) {
                    return arDrive.retryPublicArFSFileUploadByFileId({
                        wrappedFile,
                        dataTxId,
                        fileId
                    });
                }
                if (destinationFolderId) {
                    return arDrive.retryPublicArFSFileUploadByDestFolderId({
                        wrappedFile,
                        dataTxId,
                        destinationFolderId,
                        conflictResolution
                    });
                }
                throw Error('Must provide a file ID or destination folder ID in order to retry and ArFS file transaction');
            }))();
            console.log(JSON.stringify(results, null, 4));
            return error_codes_1.SUCCESS_EXIT_CODE;
        });
    })
});
