"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ardrive_core_js_1 = require("ardrive-core-js");
const __1 = require("..");
const CLICommand_1 = require("../CLICommand");
const CLICommand_2 = require("../CLICommand");
const action_1 = require("../CLICommand/action");
const error_codes_1 = require("../CLICommand/error_codes");
const parameter_declarations_1 = require("../parameter_declarations");
const get_arweave_for_url_1 = require("../utils/get_arweave_for_url");
new CLICommand_1.CLICommand({
    name: 'send-ar',
    parameters: [
        parameter_declarations_1.ArAmountParameter,
        parameter_declarations_1.DestinationAddressParameter,
        parameter_declarations_1.BoostParameter,
        parameter_declarations_1.DryRunParameter,
        ...parameter_declarations_1.WalletTypeParameters,
        parameter_declarations_1.GatewayParameter
    ],
    action: new action_1.CLIAction(function action(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const parameters = new CLICommand_2.ParametersHelper(options);
            const arAmount = parameters.getRequiredParameterValue(parameter_declarations_1.ArAmountParameter, ardrive_core_js_1.AR.from);
            const destAddress = parameters.getRequiredParameterValue(parameter_declarations_1.DestinationAddressParameter, ardrive_core_js_1.ADDR);
            const wallet = yield parameters.getRequiredWallet();
            const walletAddress = yield wallet.getAddress();
            const boost = parameters.getOptionalBoostSetting();
            console.log(`Source address: ${walletAddress}`);
            console.log(`AR amount sent: ${arAmount.toString()}`);
            console.log(`Destination address: ${destAddress}`);
            const rewardSetting = boost ? { feeMultiple: boost } : undefined;
            const dryRun = parameters.isDryRun();
            const arweave = get_arweave_for_url_1.getArweaveFromURL(parameters.getGateway());
            const walletDAO = __1.cliWalletDAOFactory(arweave);
            const arTransferResult = yield walletDAO.sendARToAddress(arAmount, wallet, destAddress, rewardSetting, dryRun, [
                { name: 'App-Name', value: __1.CLI_APP_NAME },
                { name: 'App-Version', value: __1.CLI_APP_VERSION },
                { name: 'Type', value: 'transfer' }
            ], true);
            console.log(JSON.stringify(arTransferResult, null, 4));
            return error_codes_1.SUCCESS_EXIT_CODE;
        });
    })
});
