"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ardrive_core_js_1 = require("ardrive-core-js");
const CLICommand_1 = require("../CLICommand");
const action_1 = require("../CLICommand/action");
const error_codes_1 = require("../CLICommand/error_codes");
const parameter_declarations_1 = require("../parameter_declarations");
const get_arweave_for_url_1 = require("../utils/get_arweave_for_url");
const MIN_CONFIRMATIONS = 15;
new CLICommand_1.CLICommand({
    name: 'tx-status',
    parameters: [parameter_declarations_1.TransactionIdParameter, parameter_declarations_1.ConfirmationsParameter, parameter_declarations_1.GatewayParameter],
    action: new action_1.CLIAction(function action(options) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const parameters = new CLICommand_1.ParametersHelper(options);
            const confirmations = (_a = parameters.getParameterValue(parameter_declarations_1.ConfirmationsParameter, Number)) !== null && _a !== void 0 ? _a : MIN_CONFIRMATIONS;
            const txId = parameters.getRequiredParameterValue(parameter_declarations_1.TransactionIdParameter, ardrive_core_js_1.TxID);
            const transactionsInMempool = (yield ardrive_core_js_1.fetchMempool()).map((id) => new ardrive_core_js_1.TransactionID(id));
            const pending = transactionsInMempool.some((tx) => tx.equals(txId));
            if (pending) {
                console.log(`${txId}: Pending`);
                return error_codes_1.SUCCESS_EXIT_CODE;
            }
            const arweave = get_arweave_for_url_1.getArweaveFromURL(parameters.getGateway());
            const confStatus = (yield arweave.transactions.getStatus(`${txId}`)).confirmed;
            if (!(confStatus === null || confStatus === void 0 ? void 0 : confStatus.block_height)) {
                console.log(`${txId}: Not found`);
                return error_codes_1.ERROR_EXIT_CODE;
            }
            if ((confStatus === null || confStatus === void 0 ? void 0 : confStatus.number_of_confirmations) >= confirmations) {
                console.log(`${txId}: Mined at block height ${confStatus.block_height} with ${confStatus.number_of_confirmations} confirmations`);
            }
            else {
                console.log(`${txId}: Confirming at block height ${confStatus.block_height} with ${confStatus.number_of_confirmations} confirmations`);
            }
            return error_codes_1.SUCCESS_EXIT_CODE;
        });
    })
});
