"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const CLICommand_1 = require("../CLICommand");
const parameter_declarations_1 = require("../parameter_declarations");
const prompts_1 = require("../prompts");
const error_codes_1 = require("../CLICommand/error_codes");
const action_1 = require("../CLICommand/action");
const ardrive_core_js_1 = require("ardrive-core-js");
const __1 = require("..");
const fs = __importStar(require("fs"));
const get_arweave_for_url_1 = require("../utils/get_arweave_for_url");
const temp_folder_1 = require("../utils/temp_folder");
const download_file_1 = require("../utils/download_file");
const show_progress_log_1 = require("../utils/show_progress_log");
function getFilesFromCSV(parameters) {
    var _a;
    const localCSVFile = (_a = parameters.getParameterValue(parameter_declarations_1.LocalFilesParameter_DEPRECATED)) !== null && _a !== void 0 ? _a : parameters.getParameterValue(parameter_declarations_1.LocalCSVParameter);
    if (!localCSVFile) {
        return undefined;
    }
    const localCSVFileData = fs.readFileSync(localCSVFile).toString().trim();
    const COLUMN_SEPARATOR = ',';
    const ROW_SEPARATOR = '\n';
    const csvRows = localCSVFileData.split(ROW_SEPARATOR);
    const fileParameters = csvRows.map((row) => {
        const csvFields = row.split(COLUMN_SEPARATOR).map((f) => f.trim());
        // prettier-ignore
        const [localFilePath, destinationFileName, _parentFolderId, drivePassword, _driveKey, _customContentType] = csvFields;
        const customContentType = _customContentType !== null && _customContentType !== void 0 ? _customContentType : parameters.getParameterValue(parameter_declarations_1.CustomContentTypeParameter);
        // TODO: Make CSV uploads more bulk performant
        // TODO: Make CSV work with custom metadata -- Question: How to apply this? Do we only allow metadata JSON file paths for CSV? Do we accept a stringified JSON object directly in the CSV?
        const wrappedEntity = ardrive_core_js_1.wrapFileOrFolder(localFilePath, customContentType);
        const parentFolderId = ardrive_core_js_1.EID(_parentFolderId ? _parentFolderId : parameters.getRequiredParameterValue(parameter_declarations_1.ParentFolderIdParameter));
        const driveKey = _driveKey ? new ardrive_core_js_1.EntityKey(Buffer.from(_driveKey)) : undefined;
        return {
            parentFolderId,
            wrappedEntity,
            destinationFileName,
            drivePassword: drivePassword ? drivePassword : undefined,
            driveKey
        };
    });
    return fileParameters;
}
function getFileList(parameters, parentFolderId) {
    return __awaiter(this, void 0, void 0, function* () {
        const localPaths = parameters.getParameterValue(parameter_declarations_1.LocalPathsParameter);
        if (!localPaths) {
            return undefined;
        }
        const customContentType = parameters.getParameterValue(parameter_declarations_1.CustomContentTypeParameter);
        const customMetaData = parameters.getCustomMetaData();
        const localPathsToUpload = localPaths.map((filePath) => __awaiter(this, void 0, void 0, function* () {
            const customMetaDataWithIipfsFlag = yield (function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const ipfsFlag = parameters.getParameterValue(parameter_declarations_1.IPFSParameter);
                    return ipfsFlag
                        ? yield parameters.getCustomMetaDataWithIpfsCid({ localFilePath: filePath })
                        : customMetaData;
                });
            })();
            const wrappedEntity = ardrive_core_js_1.wrapFileOrFolder(filePath, customContentType, customMetaDataWithIipfsFlag);
            return {
                parentFolderId,
                wrappedEntity
            };
        }));
        return Promise.all(localPathsToUpload);
    });
}
function getSingleFile(parameters, parentFolderId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        // NOTE: Single file is the last possible use case. Throw exception if the parameter isn't found.
        const localFilePath = (_a = parameters.getParameterValue(parameter_declarations_1.LocalFilePathParameter_DEPRECATED)) !== null && _a !== void 0 ? _a : parameters.getRequiredParameterValue(parameter_declarations_1.LocalPathParameter);
        const customContentType = parameters.getParameterValue(parameter_declarations_1.CustomContentTypeParameter);
        const customMetaData = yield (function () {
            return __awaiter(this, void 0, void 0, function* () {
                const customMetaData = parameters.getCustomMetaData();
                const ipfsFlag = parameters.getParameterValue(parameter_declarations_1.IPFSParameter);
                return ipfsFlag ? yield parameters.getCustomMetaDataWithIpfsCid({ localFilePath }) : customMetaData;
            });
        })();
        const wrappedEntity = ardrive_core_js_1.wrapFileOrFolder(localFilePath, customContentType, customMetaData);
        const singleParameter = {
            parentFolderId: parentFolderId,
            wrappedEntity,
            destinationFileName: parameters.getParameterValue(parameter_declarations_1.DestinationFileNameParameter)
        };
        return [singleParameter];
    });
}
function getRemoteFile(parameters, parentFolderId) {
    return __awaiter(this, void 0, void 0, function* () {
        const remoteFilePath = parameters.getParameterValue(parameter_declarations_1.RemotePathParameter);
        if (!remoteFilePath) {
            return undefined;
        }
        const tempFolder = temp_folder_1.getTempFolder();
        const destinationFileName = parameters.getRequiredParameterValue(parameter_declarations_1.DestinationFileNameParameter);
        const { pathToFile, contentType } = yield download_file_1.downloadFile(remoteFilePath, tempFolder, destinationFileName, (downloadProgress) => {
            if (show_progress_log_1.showProgressLog) {
                process.stderr.write(`Downloading file... ${downloadProgress.toFixed(1)}% \r`);
            }
        });
        process.stderr.clearLine(0);
        const customContentType = parameters.getParameterValue(parameter_declarations_1.CustomContentTypeParameter);
        const wrappedEntity = ardrive_core_js_1.wrapFileOrFolder(pathToFile, customContentType !== null && customContentType !== void 0 ? customContentType : contentType);
        const singleParameter = {
            parentFolderId: parentFolderId,
            wrappedEntity,
            destinationFileName: parameters.getParameterValue(parameter_declarations_1.DestinationFileNameParameter)
        };
        return [singleParameter];
    });
}
new CLICommand_1.CLICommand({
    name: 'upload-file',
    parameters: [
        parameter_declarations_1.DryRunParameter,
        parameter_declarations_1.LocalPathParameter,
        parameter_declarations_1.LocalPathsParameter,
        parameter_declarations_1.LocalCSVParameter,
        parameter_declarations_1.ParentFolderIdParameter,
        parameter_declarations_1.DestinationFileNameParameter,
        parameter_declarations_1.BoostParameter,
        parameter_declarations_1.DryRunParameter,
        parameter_declarations_1.ShouldBundleParameter,
        ...parameter_declarations_1.ConflictResolutionParams,
        ...parameter_declarations_1.DrivePrivacyParameters,
        parameter_declarations_1.CustomContentTypeParameter,
        ...parameter_declarations_1.CustomMetaDataParameters,
        parameter_declarations_1.LocalFilePathParameter_DEPRECATED,
        parameter_declarations_1.LocalFilesParameter_DEPRECATED,
        parameter_declarations_1.BoostParameter,
        parameter_declarations_1.GatewayParameter,
        parameter_declarations_1.RemotePathParameter,
        parameter_declarations_1.IPFSParameter
    ],
    action: new action_1.CLIAction(function action(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const parameters = new CLICommand_1.ParametersHelper(options);
            const filesToUpload = yield (function () {
                return __awaiter(this, void 0, void 0, function* () {
                    // Try to get the list of files to upload and their destinations, etc. from a CSV
                    const filesFromCSV = getFilesFromCSV(parameters);
                    if (filesFromCSV) {
                        return filesFromCSV;
                    }
                    // Determine list of files to upload and destinations from parameter list
                    // First check the multi-file input case
                    const parentFolderId = parameters.getRequiredParameterValue(parameter_declarations_1.ParentFolderIdParameter, ardrive_core_js_1.EID);
                    const fileList = yield getFileList(parameters, parentFolderId);
                    if (fileList) {
                        return fileList;
                    }
                    const filesFromRemote = yield getRemoteFile(parameters, parentFolderId);
                    if (filesFromRemote) {
                        return filesFromRemote;
                    }
                    // If neither the multi-file input case or csv case produced files, try the single file case (deprecated)
                    return getSingleFile(parameters, parentFolderId);
                });
            })();
            if (filesToUpload.length) {
                const wallet = yield parameters.getRequiredWallet();
                const conflictResolution = parameters.getFileNameConflictResolution();
                const shouldBundle = !!parameters.getParameterValue(parameter_declarations_1.ShouldBundleParameter);
                const remoteFilePath = parameters.getParameterValue(parameter_declarations_1.RemotePathParameter);
                const arweave = get_arweave_for_url_1.getArweaveFromURL(parameters.getGateway());
                const arDrive = __1.cliArDriveFactory({
                    wallet,
                    feeMultiple: parameters.getOptionalBoostSetting(),
                    dryRun: parameters.isDryRun(),
                    shouldBundle,
                    arweave
                });
                const uploadStats = yield (() => __awaiter(this, void 0, void 0, function* () {
                    const uploadStats = [];
                    for (const { parentFolderId, wrappedEntity, destinationFileName, driveKey, drivePassword } of filesToUpload) {
                        uploadStats.push({
                            wrappedEntity,
                            driveKey: (driveKey !== null && driveKey !== void 0 ? driveKey : (yield parameters.getIsPrivate()))
                                ? yield parameters.getDriveKey({
                                    driveId: yield arDrive.getDriveIdForFolderId(parentFolderId),
                                    drivePassword,
                                    useCache: true
                                })
                                : undefined,
                            destFolderId: parentFolderId,
                            destName: destinationFileName
                        });
                    }
                    return uploadStats;
                }))();
                const results = yield arDrive.uploadAllEntities({
                    entitiesToUpload: uploadStats,
                    conflictResolution,
                    prompts: prompts_1.fileAndFolderUploadConflictPrompts
                });
                if (remoteFilePath && results.created[0].type === 'file') {
                    // TODO: Include ArFSRemoteFileToUpload functionality in ArDrive Core
                    // TODO: Account for bulk remote path uploads in the future
                    results.created[0].sourceUri = remoteFilePath;
                }
                console.log(JSON.stringify(results, null, 4));
                temp_folder_1.cleanUpTempFolder();
                return error_codes_1.SUCCESS_EXIT_CODE;
            }
            console.log(`No files to upload`);
            return error_codes_1.ERROR_EXIT_CODE;
        });
    })
});
