"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getNewNameFromRenamePrompt = exports.fileAndFolderUploadConflictPrompts = exports.fileUploadConflictPrompts = exports.folderToFolderNameConflict = exports.folderToFileNameConflict = exports.fileToFolderNameConflict = exports.fileToFileNameConflict = void 0;
const prompts_1 = __importDefault(require("prompts"));
const ardrive_core_js_1 = require("ardrive-core-js");
const fileToFileNameConflict = ({ fileId, fileName, hasSameLastModifiedDate, namesWithinDestFolder: namesWithinFolder }) => __awaiter(void 0, void 0, void 0, function* () {
    console.log(''); // Add empty line for readability
    const { resolution } = yield prompts_1.default({
        type: 'select',
        name: 'resolution',
        message: `Destination folder has a file to file name conflict!
		\nFile name: ${fileName}\nFile ID: ${fileId}\nThis file has a ${hasSameLastModifiedDate ? 'MATCHING' : 'DIFFERENT'} last modified date
		\nPlease select how to proceed:\n`,
        choices: [promptChoices.replaceAsRevision, promptChoices.uploadFileWithNewName, promptChoices.skipFileUpload]
    });
    if (resolution === ardrive_core_js_1.skipOnConflicts) {
        return { resolution: ardrive_core_js_1.skipOnConflicts };
    }
    if (resolution === ardrive_core_js_1.replaceOnConflicts) {
        return { resolution: ardrive_core_js_1.replaceOnConflicts };
    }
    if (resolution === ardrive_core_js_1.renameOnConflicts) {
        const newFileName = yield getNewNameFromRenamePrompt(fileName, namesWithinFolder, fileNamePrompt);
        return { resolution: ardrive_core_js_1.renameOnConflicts, newFileName };
    }
    throw new Error(conflictInterruptedError);
});
exports.fileToFileNameConflict = fileToFileNameConflict;
const fileToFolderNameConflict = ({ folderId, folderName, namesWithinDestFolder: namesWithinFolder }) => __awaiter(void 0, void 0, void 0, function* () {
    console.log(''); // Add empty line for readability
    const { resolution } = yield prompts_1.default({
        type: 'select',
        name: 'resolution',
        message: `Destination folder has a file to folder name conflict!
		\nFolder name: ${folderName}\nFolderID: ${folderId}
		\nPlease select how to proceed:\n`,
        choices: [promptChoices.uploadFileWithNewName, promptChoices.skipFileUpload]
    });
    if (resolution === ardrive_core_js_1.skipOnConflicts) {
        return { resolution: ardrive_core_js_1.skipOnConflicts };
    }
    if (resolution === ardrive_core_js_1.renameOnConflicts) {
        const newFileName = yield getNewNameFromRenamePrompt(folderName, namesWithinFolder, fileNamePrompt);
        return { resolution: ardrive_core_js_1.renameOnConflicts, newFileName };
    }
    throw new Error(conflictInterruptedError);
});
exports.fileToFolderNameConflict = fileToFolderNameConflict;
const folderToFileNameConflict = ({ fileId, fileName, namesWithinDestFolder: namesWithinFolder }) => __awaiter(void 0, void 0, void 0, function* () {
    console.log(''); // Add empty line for readability
    const { resolution } = yield prompts_1.default({
        type: 'select',
        name: 'resolution',
        message: `Destination folder has a folder to file name conflict!
		\nFile name: ${fileName}\nFile ID: ${fileId}
		\nPlease select how to proceed:\n`,
        choices: [promptChoices.createFolderWithNewName, promptChoices.skipFolderAndContents]
    });
    if (resolution === ardrive_core_js_1.skipOnConflicts) {
        return { resolution: ardrive_core_js_1.skipOnConflicts };
    }
    if (resolution === ardrive_core_js_1.renameOnConflicts) {
        const newFolderName = yield getNewNameFromRenamePrompt(fileName, namesWithinFolder, folderNamePrompt);
        return { resolution: ardrive_core_js_1.renameOnConflicts, newFolderName };
    }
    throw new Error(conflictInterruptedError);
});
exports.folderToFileNameConflict = folderToFileNameConflict;
const folderToFolderNameConflict = ({ folderId, folderName, namesWithinDestFolder: namesWithinFolder }) => __awaiter(void 0, void 0, void 0, function* () {
    console.log(''); // Add empty line for readability
    const { resolution } = yield prompts_1.default({
        type: 'select',
        name: 'resolution',
        message: `Destination folder has a folder to folder name conflict!
		\nFolder name: ${folderName}\nFolder ID: ${folderId}
		\nPlease select how to proceed:\n`,
        choices: [
            promptChoices.useExistingFolder,
            promptChoices.createFolderWithNewName,
            promptChoices.skipFolderAndContents
        ]
    });
    if (resolution === ardrive_core_js_1.skipOnConflicts) {
        return { resolution: ardrive_core_js_1.skipOnConflicts };
    }
    if (resolution === ardrive_core_js_1.useExistingFolder) {
        return { resolution: ardrive_core_js_1.useExistingFolder };
    }
    if (resolution === ardrive_core_js_1.renameOnConflicts) {
        const newFolderName = yield getNewNameFromRenamePrompt(folderName, namesWithinFolder, folderNamePrompt);
        return { resolution: ardrive_core_js_1.renameOnConflicts, newFolderName };
    }
    throw new Error(conflictInterruptedError);
});
exports.folderToFolderNameConflict = folderToFolderNameConflict;
const conflictInterruptedError = 'Name conflict prompt was interrupted or could not be resolved!';
const fileNamePrompt = () => prompts_1.default({
    type: 'text',
    name: 'newName',
    message: 'Enter new file name'
});
const folderNamePrompt = () => prompts_1.default({
    type: 'text',
    name: 'newName',
    message: 'Enter new folder name'
});
const promptChoices = {
    useExistingFolder: { title: 'Re-use existing folder', value: ardrive_core_js_1.useExistingFolder },
    createFolderWithNewName: { title: 'Create folder with a different name', value: ardrive_core_js_1.renameOnConflicts },
    uploadFileWithNewName: { title: 'Upload with a different file name', value: ardrive_core_js_1.renameOnConflicts },
    skipFolderAndContents: { title: 'Skip uploading folder and all of its contents', value: ardrive_core_js_1.skipOnConflicts },
    skipFileUpload: { title: 'Skip this file upload', value: ardrive_core_js_1.skipOnConflicts },
    replaceAsRevision: { title: 'Replace as new file revision', value: ardrive_core_js_1.replaceOnConflicts }
};
exports.fileUploadConflictPrompts = {
    fileToFileNameConflict: exports.fileToFileNameConflict,
    fileToFolderNameConflict: exports.fileToFolderNameConflict
};
exports.fileAndFolderUploadConflictPrompts = Object.assign(Object.assign({}, exports.fileUploadConflictPrompts), { folderToFolderNameConflict: exports.folderToFolderNameConflict,
    folderToFileNameConflict: exports.folderToFileNameConflict });
/** Shared utility function to resolve and display new conflicts during rename */
function getNewNameFromRenamePrompt(conflictingName, namesWithinFolder, prompt) {
    return __awaiter(this, void 0, void 0, function* () {
        let nameFromRename = undefined;
        while (!nameFromRename) {
            const { newName } = yield prompt();
            if (newName === undefined) {
                throw new Error(conflictInterruptedError);
            }
            // Repeat the prompt if name is an empty string, remains unchanged, or conflicts with another name
            if (newName === '') {
                console.log('Please enter a new name...');
                continue;
            }
            if (newName === conflictingName) {
                console.log('That is the same file name, please choose a new name!');
                continue;
            }
            if (namesWithinFolder.includes(newName)) {
                console.log('That name also conflicts with name in the destination file, choose a non-conflicting name!');
                continue;
            }
            nameFromRename = newName;
        }
        return nameFromRename;
    });
}
exports.getNewNameFromRenamePrompt = getNewNameFromRenamePrompt;
