"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.downloadFile = void 0;
const fs = __importStar(require("fs"));
const path_1 = __importDefault(require("path"));
const axios_1 = __importDefault(require("axios"));
const util_1 = __importDefault(require("util"));
const stream_1 = __importDefault(require("stream"));
const pipeline = util_1.default.promisify(stream_1.default.pipeline);
/**
 * Downloads file from remote HTTP[S] host and puts its contents to the
 * specified location.
 * @param url URL of the file to download.
 * @param destinationPath Path to the destination file.
 * @param destinationFileName The file name.
 */
function downloadFile(url, destinationPath, destinationFileName, downloadProgressCallback) {
    return __awaiter(this, void 0, void 0, function* () {
        const pathToFile = path_1.default.join(destinationPath, destinationFileName);
        const writer = fs.createWriteStream(pathToFile);
        try {
            const { data, headers } = yield axios_1.default({
                method: 'get',
                url: url,
                responseType: 'stream'
            });
            const totalLength = headers['content-length'];
            const contentType = headers['content-type'];
            let downloadedLength = 0;
            data.on('data', (chunk) => {
                downloadedLength += chunk.length;
                const downloadProgressPct = totalLength > 0 ? (downloadedLength / totalLength) * 100 : 0;
                downloadProgressCallback && downloadProgressCallback(downloadProgressPct);
            });
            yield pipeline(data, writer);
            return { pathToFile, contentType };
        }
        catch (error) {
            writer.close();
            throw new Error(`Failed to download file from remote path ${url}: ${error.message}`);
        }
    });
}
exports.downloadFile = downloadFile;
