"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getOutputFolderPathAndName = exports.getOutputFilePathAndName = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
/**
 * For testing purposes.
 * This object is the default value of the last argument of getOutputFilePathAndName
 */
const fsStatSyncAndPathResolve = {
    statSync: fs_1.statSync,
    resolve: path_1.resolve
};
/**
 * Resolves the path, verifies its existance and type to conditionally return its dir and file name
 * @param {string} destOutputPath - the path from where to extract the dir path and name
 * @returns {FilePathAndName} - the directory where to put the file and the file name (which is undefined when the provided destOutputPath is a directory)
 */
function getOutputFilePathAndName(destOutputPath, fsStatSyncAndPathResolveWrapper = fsStatSyncAndPathResolve) {
    const resolvedOutputPath = fsStatSyncAndPathResolveWrapper.resolve(destOutputPath);
    const outputDirname = path_1.dirname(resolvedOutputPath);
    const outputBasename = path_1.basename(resolvedOutputPath);
    try {
        const outputPathStats = fsStatSyncAndPathResolveWrapper.statSync(resolvedOutputPath);
        // the destination does exist
        if (outputPathStats.isDirectory()) {
            // and is a directory
            return [resolvedOutputPath];
        }
        else if (outputPathStats.isFile()) {
            // as an existing file
            return [outputDirname, outputBasename];
        }
    }
    catch (e) {
        // the destination doesn't exist
        const outputParentPathStats = fsStatSyncAndPathResolveWrapper.statSync(outputDirname);
        // the parent exists
        if (outputParentPathStats.isDirectory()) {
            return [outputDirname, outputBasename];
        }
        throw new Error(`The path ${outputDirname} is not a directory!`);
    }
    throw new Error(`The destination is neither a folder nor a file!`);
}
exports.getOutputFilePathAndName = getOutputFilePathAndName;
/**
 * Resolves the path, verifies its existance and type to conditionally return its dir and file name
 * @param {string} destOutputPath - the path from where to extract the dir path and name
 * @param fsStatSyncAndPathResolveWrapper - for testing purposes it wraps the fs methods
 * @returns {FilePathAndName} - the directory where to put the file and the file name (which is undefined when the provided destOutputPath is a directory)
 */
function getOutputFolderPathAndName(destOutputPath, fsStatSyncAndPathResolveWrapper = fsStatSyncAndPathResolve) {
    const resolvedOutputPath = fsStatSyncAndPathResolveWrapper.resolve(destOutputPath);
    const outputDirname = path_1.dirname(resolvedOutputPath);
    const outputBasename = path_1.basename(resolvedOutputPath);
    try {
        const outputPathStats = fsStatSyncAndPathResolveWrapper.statSync(resolvedOutputPath);
        // the destination does exist
        if (outputPathStats.isDirectory()) {
            // TODO: check case sensitivity conflicts here
            // and is a directory
            return [resolvedOutputPath];
        }
    }
    catch (e) {
        // the destination doesn't exist
        const outputParentPathStats = fsStatSyncAndPathResolveWrapper.statSync(outputDirname);
        // the parent exists
        if (outputParentPathStats.isDirectory()) {
            return [outputDirname, outputBasename];
        }
        // TODO: handle the recoverable error case for `ENAMETOOLONG`
        throw new Error("The destination path doesn't exist"); // TODO: suggest to use the `--create-parents` flag
    }
    throw new Error(`The destination isn't a folder!`);
}
exports.getOutputFolderPathAndName = getOutputFolderPathAndName;
