"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cleanUpTempFolder = exports.getTempFolder = void 0;
const fs = __importStar(require("fs"));
const os = __importStar(require("os"));
const path_1 = __importDefault(require("path"));
const isWindows = os.platform() === 'win32';
function getOrCreateTempBaseFolder() {
    var _a, _b;
    if (isWindows) {
        return (_a = getValidWindowsTempPath()) !== null && _a !== void 0 ? _a : getManualWindowsTempPath();
    }
    else {
        return (_b = getValidUnixTempPath()) !== null && _b !== void 0 ? _b : getManualUnixTempPath();
    }
}
function getValidWindowsTempPath() {
    var _a;
    const envTempFolder = (_a = process.env['TEMP']) !== null && _a !== void 0 ? _a : process.env['TMP'];
    if (envTempFolder) {
        if (fs.existsSync(envTempFolder)) {
            return envTempFolder;
        }
        else {
            const userProfile = process.env['USERPROFILE'];
            if (userProfile) {
                const tempPath = path_1.default.join(userProfile, 'AppData', 'Local', 'Temp');
                if (fs.existsSync(tempPath)) {
                    return tempPath;
                }
            }
        }
    }
    return null;
}
function getValidUnixTempPath() {
    var _a, _b;
    const envTempFolder = (_b = (_a = process.env['TMPDIR']) !== null && _a !== void 0 ? _a : process.env['TMP']) !== null && _b !== void 0 ? _b : process.env['TEMP'];
    if (envTempFolder) {
        if (fs.existsSync(envTempFolder)) {
            return envTempFolder;
        }
        else {
            const tempPath = '/tmp';
            if (fs.existsSync(tempPath)) {
                return tempPath;
            }
        }
    }
    return null;
}
function getManualWindowsTempPath() {
    return path_1.default.join(os.homedir(), 'ardrive-temp');
}
function getManualUnixTempPath() {
    return path_1.default.join(os.homedir(), '.ardrive');
}
function platformTempFolder() {
    const tempBaseFolder = getOrCreateTempBaseFolder();
    return path_1.default.join(tempBaseFolder, 'ardrive-downloads');
}
/**
 * Gets a folder path for storing temporary files.
 */
function getTempFolder() {
    const tempFolderPath = platformTempFolder();
    if (fs.existsSync(tempFolderPath)) {
        return tempFolderPath;
    }
    fs.mkdirSync(tempFolderPath, { recursive: true });
    return tempFolderPath;
}
exports.getTempFolder = getTempFolder;
function cleanUpTempFolder() {
    const tempFolderPath = platformTempFolder();
    if (fs.existsSync(tempFolderPath)) {
        fs.rmdirSync(tempFolderPath, { recursive: true });
    }
}
exports.cleanUpTempFolder = cleanUpTempFolder;
